@@ -1,32 +1,38 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
-* ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
-* -*- mode: c, c-basic-offset: 4 -*- */
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright (C) Centeris Corporation 2004-2007
- * Copyright (C) Likewise Software    2007-2008
+ * Copyright Likewise Software    2004-2008
  * All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as 
- * published by the Free Software Foundation; either version 2.1 of 
- * the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU Lesser General Public 
- * License along with this program.  If not, see 
- * <http://www.gnu.org/licenses/>.
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
  */
 
+#include "config.h"
 #include "ctshell.h"
 #include "ctarray.h"
 #include <sys/types.h>
 #include <sys/wait.h>
-#include "config/config.h"
 
 #define SIGIL '%'
 
@@ -337,6 +343,7 @@ AppendVariable(Command* result, struct CTShellVar* var, BOOLEAN inDouble)
     char* str = NULL;
     char const * const * ptr = NULL;
     Pipe* ppipe = NULL;
+    int fd;
     
     if (!var)
         BAIL_ON_CENTERIS_ERROR(ceError = CENTERROR_SHELL_VARIABLE_UNKNOWN);
@@ -369,7 +376,9 @@ AppendVariable(Command* result, struct CTShellVar* var, BOOLEAN inDouble)
             BAIL_ON_CENTERIS_ERROR(ceError = CTMapSystemError(errno));
         BAIL_ON_CENTERIS_ERROR(ceError = CTStringBufferConstruct(&ppipe->buffer));
 
-        BAIL_ON_CENTERIS_ERROR(ceError = CTAllocateStringPrintf(&str, "&%i", ppipe->fds[1]));
+        fd = result->pipes.size - 1 + 3;
+
+        BAIL_ON_CENTERIS_ERROR(ceError = CTAllocateStringPrintf(&str, "&%i %i>&-", fd, fd));
         BAIL_ON_CENTERIS_ERROR(ceError = CTStringBufferAppend(&result->buffer, str));
 
         ppipe = NULL;
@@ -491,6 +500,7 @@ ConstructShellCommand(const char* format, Command *result)
             break;
         }
     }
+
 error:
     CT_SAFE_FREE_STRING(variable);
     return ceError;
@@ -523,6 +533,14 @@ ExecuteShellCommand(char * const envp[], Command* command)
                     abort();
             }
         }
+
+        // Remap fds to lowest numbers possible
+        for (i = 0; i < command->pipes.size; i++)
+        {
+            dup2(pipes[i]->fds[1], i+3);
+            close(pipes[i]->fds[1]);
+        }
+
 	if (envp)
 	    execle("/bin/sh", (const char*) "sh", (const char*) "-c", (const char*) command->buffer.data, (const char*) NULL, envp);
 	else
@@ -531,7 +549,7 @@ ExecuteShellCommand(char * const envp[], Command* command)
     }
     else
     {
-        int status;
+        int status = 0;
         while (1)
         {
             // Parent
