@@ -1,25 +1,31 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
-* ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
-* -*- mode: c, c-basic-offset: 4 -*- */
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright (C) Centeris Corporation 2004-2007
- * Copyright (C) Likewise Software    2007-2008
+ * Copyright Likewise Software    2004-2008
  * All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as 
- * published by the Free Software Foundation; either version 2.1 of 
- * the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU Lesser General Public 
- * License along with this program.  If not, see 
- * <http://www.gnu.org/licenses/>.
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
  */
 
 #include "domainjoin.h"
@@ -116,15 +122,13 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 #   Red Hat Enterprise Linux ES release 4 (Nahant Update 1)
                 #   Red Hat Linux Advanced Server release 2.1AS (Pensacola)
                 #   Red Hat Enterprise Linux Client release 5 (Tikanga)
+                #   Red Hat Enterprise Linux Server release 5.3 (Tikanga)
                 # In addition, Oracle Linux reports itself as:
                 #   Enterprise Linux Enterprise Linux AS release 4 (October Update 5)
                 */
                 //Find a matching distro name
-                "^[[:space:]]*((Red Hat)|(Enterprise Linux)) ((Enterprise Linux)|(Linux (Advanced|Enterprise) Server))[[:space:]]+(AS |ES |Client )?"
-                //Get the version number, but strip the minor version if it is
-                //present (RHEL 2 has one). Also remove the AS or ES
-                //suffix if it is present.
-                "release ([[:digit:]]+)(\\.[[:digit:]]+)?(AS|ES)?([[:space:]]+\\(.*\\))?[[:space:]]*$",
+                "^[[:space:]]*((Red Hat)|(Enterprise Linux)) ((Enterprise Linux)|(Linux (Advanced|Enterprise) Server))[[:space:]]+(AS |ES |Client |Server )?"
+                "release ([[:digit:]]+(\\.[[:digit:]]+)?(AS|ES)?)",
                 9,
                 1
             },
@@ -157,9 +161,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 # The format of the line is something like:
                 #   CentOS release 4.x (Final)
                 */
-                "^[[:space:]]*CentOS release ([[:digit:]]+)"
-                //Trim off the minor version number
-                "(\\.[[:digit:]]+)?",
+                "^[[:space:]]*CentOS release ([[:digit:]]+(\\.[[:digit:]]+)?)",
                 1,
                 1
             },
@@ -180,14 +182,14 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
             {
                 DISTRO_SLES,
                 "/etc/SuSE-release",
-                "^[[:space:]]*SUSE LINUX Enterprise Server ([[:digit:]]+)[[:space:]]+",
+                "^[[:space:]]*SUSE LINUX Enterprise Server ([[:digit:]]+( SP[[:digit:]]+)?)",
                 1,
                 0
             },
             {
                 DISTRO_SLED,
                 "/etc/SuSE-release",
-                "^[[:space:]]*SUSE LINUX Enterprise Desktop ([[:digit:]]+)[[:space:]]+",
+                "^[[:space:]]*SUSE LINUX Enterprise Desktop ([[:digit:]]+( SP[[:digit:]]+)?)",
                 1,
                 0
             },
@@ -317,6 +319,11 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 GCE(ceError = CTStrdup(temp, &info->version));
             }
             break;
+        case OS_FREEBSD:
+            info->distro = DISTRO_FREEBSD;
+            GCE(ceError = CTAllocateStringPrintf(&info->version,
+                        "%s", unameStruct.release));
+            break;
         default:
             info->distro = DISTRO_UNKNOWN;
         }
@@ -473,8 +480,8 @@ struct
     { OS_DARWIN, "Darwin"},
     { OS_DARWIN, "OsX" },
     { OS_HPUX, "HP-UX"},
-    { OS_HPUX, "HPUX" },
     { OS_LINUX, "Linux" },
+    { OS_FREEBSD, "FreeBSD" },
 };
 
 OSType DJGetOSFromString(const char *str)
@@ -508,10 +515,8 @@ struct
     { DISTRO_AIX, "AIX" },
     { DISTRO_SUNOS, "Solaris" },
     { DISTRO_SUNOS, "SunOS" },
-    { DISTRO_DARWIN, "Darwin"},
-    { DISTRO_DARWIN, "OsX" },
+    { DISTRO_DARWIN, "Mac OS X"},
     { DISTRO_HPUX, "HP-UX"},
-    { DISTRO_HPUX, "HPUX" },
     { DISTRO_RHEL, "RHEL" },
     { DISTRO_REDHAT, "Redhat" },
     { DISTRO_FEDORA, "Fedora" },
@@ -522,6 +527,7 @@ struct
     { DISTRO_SLED, "SLED" },
     { DISTRO_UBUNTU, "Ubuntu" },
     { DISTRO_DEBIAN, "Debian" },
+    { DISTRO_FREEBSD, "FreeBSD" },
 };
 
 DistroType DJGetDistroFromString(const char *str)
@@ -593,3 +599,80 @@ void DJFreeDistroInfo(DistroInfo *info)
     if(info != NULL)
         CT_SAFE_FREE_STRING(info->version);
 }
+
+CENTERROR DJGetLikewiseVersion(PSTR *version, PSTR *build, PSTR *revision)
+{
+    FILE *versionFile = NULL;
+    PSTR line = NULL; 
+    BOOLEAN isEndOfFile = FALSE;
+    CENTERROR ceError = CENTERROR_SUCCESS;
+
+    PSTR _version = NULL;
+    PSTR _build = NULL;
+    PSTR _revision = NULL;
+
+    *version = NULL;
+    *build = NULL;
+    *revision = NULL;
+
+#ifdef MINIMAL_JOIN
+    GCE(ceError = CTOpenFile(LOCALSTATEDIR "/VERSION", "r", &versionFile));
+#else
+    GCE(ceError = CTOpenFile(PREFIXDIR "/data/VERSION", "r", &versionFile));
+#endif
+
+    while (TRUE)
+    {
+        GCE(ceError = CTReadNextLine(versionFile, &line, &isEndOfFile));
+        if (isEndOfFile)
+            break;
+        CTStripWhitespace(line);
+        if (!strncmp(line, "VERSION=", sizeof("VERSION=") - 1))
+        {
+            GCE(ceError = CTStrdup(line + sizeof("VERSION=") - 1,
+                        &_version));
+        }
+        else if (!strncmp(line, "BUILD=", sizeof("BUILD=") - 1))
+        {
+            GCE(ceError = CTStrdup(line + sizeof("BUILD=") - 1,
+                    &_build));
+        }
+        else if (!strncmp(line, "REVISION=", sizeof("REVISION=") - 1))
+        {
+            GCE(ceError = CTStrdup(line + sizeof("REVISION=") - 1,
+                    &_revision));
+        }
+    }
+
+    if (_version == NULL)
+    {
+        GCE(ceError = CTStrdup("unknown", &_version));
+    }
+    if (_build == NULL)
+    {
+        GCE(ceError = CTStrdup("unknown", &_build));
+    }
+    if (_revision == NULL)
+    {
+        GCE(ceError = CTStrdup("unknown", &_revision));
+    }
+
+    GCE(ceError = CTSafeCloseFile(&versionFile));
+
+    *version = _version;
+    *build = _build;
+    *revision = _revision;
+    _version = NULL;
+    _build = NULL;
+    _revision = NULL;
+    
+cleanup:
+
+    CTSafeCloseFile(&versionFile);
+    CT_SAFE_FREE_STRING(line);
+    CT_SAFE_FREE_STRING(_version);
+    CT_SAFE_FREE_STRING(_build);
+    CT_SAFE_FREE_STRING(_revision);
+
+    return ceError;
+}
