@@ -1,33 +1,40 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
-* ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
-* -*- mode: c, c-basic-offset: 4 -*- */
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright (C) Centeris Corporation 2004-2007
- * Copyright (C) Likewise Software    2007-2008
+ * Copyright Likewise Software    2004-2008
  * All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as 
- * published by the Free Software Foundation; either version 2.1 of 
- * the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU Lesser General Public 
- * License along with this program.  If not, see 
- * <http://www.gnu.org/licenses/>.
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
  */
 
-
 #include "domainjoin.h"
 #include "ctarray.h"
 #include "ctstrutils.h"
+#include "ctshell.h"
 #include "djstr.h"
 #include "djauthinfo.h"
+#include <lsa/lsa.h>
 
 #define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
 
@@ -199,7 +206,7 @@ static CENTERROR CopyEntry(const Krb5Entry *source, Krb5Entry **copy)
     Krb5Entry *created = NULL;
     Krb5Entry *copiedChild = NULL;
     size_t i;
-    GCE(ceError = CTAllocateMemory(sizeof(*created), (void **)&created));
+    GCE(ceError = CTAllocateMemory(sizeof(*created), (void **) (void*)&created));
     GCE(ceError = CTDupOrNullStr(source->leadingWhiteSpace, &created->leadingWhiteSpace));
     GCE(ceError = CTCopyTokenContents(&created->name, &source->name));
     GCE(ceError = CTCopyTokenContents(&created->beginSeparator, &source->beginSeparator));
@@ -253,7 +260,7 @@ static CENTERROR ParseLine(Krb5Entry **parent, const char *linestr, const char *
     Krb5Entry *line = NULL;
     BOOLEAN expectChildren = FALSE;
 
-    GCE(ceError = CTAllocateMemory(sizeof(*line), (void **)&line));
+    GCE(ceError = CTAllocateMemory(sizeof(*line), (void**) (void*)&line));
 
     /* Find the leading whitespace in the line */
     token_start = pos;
@@ -343,7 +350,7 @@ static CENTERROR ParseLine(Krb5Entry **parent, const char *linestr, const char *
         }
     }
 
-    GCE(ceError = AddChildNode(*parent, line));
+    GCE(ceError = InsertChildNode(*parent, (*parent)->subelements.size, line));
     if(expectChildren)
     {
         *parent = line;
@@ -485,9 +492,9 @@ static CENTERROR CreateValueNode(Krb5Entry *conf, int depth, const char *name, c
     if(existing == NULL)
     {
         //Couldn't find a template. Got to create it from scratch
-        GCE(ceError = CTAllocateMemory(sizeof(*created), (void **)&created));
+        GCE(ceError = CTAllocateMemory(sizeof(*created), (void**) (void*)&created));
         //Use one space to indent everything past the stanza level
-        GCE(ceError = CTAllocateMemory(depth, (void **)&created->leadingWhiteSpace));
+        GCE(ceError = CTAllocateMemory(depth + 1, (void**) (void*)&created->leadingWhiteSpace));
         memset(created->leadingWhiteSpace, ' ', depth);
         created->leadingWhiteSpace[depth] = 0;
         GCE(ceError = CTStrdup(" ", &created->name.trailingSeparator));
@@ -527,9 +534,9 @@ static CENTERROR CreateGroupNode(Krb5Entry *conf, int depth, const char *name, K
     if(existing == NULL)
     {
         //Couldn't find a template. Got to create it from scratch
-        GCE(ceError = CTAllocateMemory(sizeof(*created), (void **)&created));
+        GCE(ceError = CTAllocateMemory(sizeof(*created), (void**) (void*)&created));
         //Use one space to indent everything past the stanza level
-        GCE(ceError = CTAllocateMemory(depth, (void **)&created->leadingWhiteSpace));
+        GCE(ceError = CTAllocateMemory(depth + 1, (void**) (void*)&created->leadingWhiteSpace));
         memset(created->leadingWhiteSpace, ' ', depth);
         created->leadingWhiteSpace[depth] = 0;
         GCE(ceError = CTStrdup(" ", &created->name.trailingSeparator));
@@ -568,7 +575,7 @@ static CENTERROR CreateStanzaNode(Krb5Entry *conf, const char *name, Krb5Entry *
     if(existing == NULL)
     {
         //Couldn't find a template. Got to create it from scratch
-        GCE(ceError = CTAllocateMemory(sizeof(*created), (void **)&created));
+        GCE(ceError = CTAllocateMemory(sizeof(*created), (void**) (void*)&created));
     }
     else
     {
@@ -827,22 +834,6 @@ static void FreeDomainMappings(DynamicArray *mappings)
     CTArrayFree(mappings);
 }
 
-static ssize_t
-FindMapping(DynamicArray *mappings,
-    PSTR shortName)
-{
-    size_t i;
-    for(i = 0; i < mappings->size; i++)
-    {
-        DomainMapping *current = ((DomainMapping *)mappings->data) + i;
-        if(!strcmp(current->shortName, shortName))
-        {
-            return i;
-        }
-    }
-    return -1;
-}
-
 static CENTERROR
 GatherDomainMappings(
     DynamicArray *mappings,
@@ -851,7 +842,11 @@ GatherDomainMappings(
 {
     CENTERROR ceError = CENTERROR_SUCCESS;
     DomainMapping add;
-    char *commandOutput = NULL;
+    HANDLE hLsa = NULL;
+    PLSASTATUS pStatus = NULL;
+    size_t providerIndex = 0;
+    size_t domainIndex = 0;
+
     memset(mappings, 0, sizeof(*mappings));
     memset(&add, 0, sizeof(add));
 
@@ -860,136 +855,139 @@ GatherDomainMappings(
     GCE(ceError = CTArrayAppend(mappings, sizeof(add), &add, 1));
     memset(&add, 0, sizeof(add));
 
-    //This command requires that the computer is joined to the domain, but
-    //the auth daemon does not need to be running
-    ceError = CTCaptureOutputWithStderr(
-        PREFIXDIR "/bin/lwinet ads trusts",
-        TRUE,
-        &commandOutput);
-    if(ceError == CENTERROR_SUCCESS)
+    ceError = LsaOpenServer(&hLsa);
+    if (ceError == ERROR_FILE_NOT_FOUND || ceError == LW_ERROR_ERRNO_ECONNREFUSED)
     {
-        char *linesaveptr;
-        char *line;
-        line = strtok_r(commandOutput, "\r\n", &linesaveptr);
-        while(line != NULL)
+        DJ_LOG_INFO("Unable to get trust list because lsass is not running");
+        ceError = 0;
+        goto cleanup;
+    }
+    GCE(ceError);
+
+    GCE(ceError = LsaGetStatus(hLsa, &pStatus));
+
+    for (providerIndex = 0; providerIndex < pStatus->dwCount; providerIndex++)
+    {
+        PLSA_AUTH_PROVIDER_STATUS provider = &pStatus->pAuthProviderStatusList[providerIndex];
+        for (domainIndex = 0; domainIndex < provider->dwNumTrustedDomains; domainIndex++)
         {
-            //The output is in the form:
-            //<trustnum> <short name> <num> <hexnum> <hexnum> <hexnum> <SID> <GUID> <long name>
-            //e.g.:
-            //3 CORP                0 0x1d 0x02 0x00 S-1-5-21-418081286-1191099226-2202501032  {19df18bd-f2f4-453a-82fd-63b02c896ae1} corp.centeris.com
-            char *tokensaveptr;
-            char *shortName;
-            char *longName;
-            CT_SAFE_FREE_STRING(add.shortName);
-            CT_SAFE_FREE_STRING(add.longName);
-            //trustnum
-            strtok_r(line, " \t", &tokensaveptr);
-            shortName = strtok_r(NULL, " \t", &tokensaveptr);
-            if(shortName != NULL)
-                GCE(ceError = CTStrdup(shortName, &add.shortName));
-            //<num>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            //<hexnum>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            //<hexnum>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            //<hexnum>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            //<SID>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            //<GUID>
-            strtok_r(NULL, " \t", &tokensaveptr);
-            longName = strtok_r(NULL, " \t", &tokensaveptr);
-            if(longName != NULL)
-                GCE(ceError = CTStrdup(longName, &add.longName));
-            CTStripWhitespace(add.shortName);
-            CTStripWhitespace(add.longName);
-
-            if(!IsNullOrEmptyString(add.shortName) && !IsNullOrEmptyString(add.longName))
-            {
-                DJ_LOG_INFO("Lwinet found trust '%s' -> '%s'",
-                        add.shortName,
-                        add.longName);
-                if( FindMapping(mappings, add.shortName) == -1)
-                {
-                    GCE(ceError = CTArrayAppend(mappings,
-                                sizeof(add), &add, 1));
-                    memset(&add, 0, sizeof(add));
-                }
-            }
-
-            line = strtok_r(NULL, "\r\n", &linesaveptr);
+            GCE(ceError = CTStrdup(provider->
+                        pTrustedDomainInfoArray[domainIndex].pszDnsDomain,
+                        &add.longName));
+            GCE(ceError = CTStrdup(provider->
+                        pTrustedDomainInfoArray[domainIndex].pszNetbiosDomain,
+                        &add.shortName));
+            GCE(ceError = CTArrayAppend(mappings, sizeof(add), &add, 1));
+            memset(&add, 0, sizeof(add));
         }
     }
-    else if(ceError == CENTERROR_COMMAND_FAILED)
-    {
-         DJ_LOG_INFO("Failed to run lwinet ads trusts. This is expected if not yet joined to the domain");
-         ceError = CENTERROR_SUCCESS;
-    }
-    else
-        GCE(ceError);
-
-    CT_SAFE_FREE_STRING(commandOutput);
-    //This command requires that the auth daemon is running
-    ceError = CTCaptureOutputWithStderr(
-        PREFIXDIR "/bin/lwiinfo --details -m",
-        TRUE,
-        &commandOutput);
-    if(ceError == CENTERROR_SUCCESS)
-    {
-        char *linesaveptr;
-        char *line;
-        line = strtok_r(commandOutput, "\r\n", &linesaveptr);
-        while(line != NULL)
-        {
-            //The output is in the form:
-            //<short name>, <long name>, <sid>
-            //e.g.:
-            //CORP, corp.centeris.com, S-1-5-21-418081286-1191099226-2202501032
-            char *tokensaveptr;
-            char *shortName;
-            char *longName;
-            CT_SAFE_FREE_STRING(add.shortName);
-            CT_SAFE_FREE_STRING(add.longName);
-            shortName = strtok_r(line, ",", &tokensaveptr);
-            if(shortName != NULL)
-                GCE(ceError = CTStrdup(shortName, &add.shortName));
-            longName = strtok_r(NULL, ",", &tokensaveptr);
-            if(longName != NULL)
-                GCE(ceError = CTStrdup(longName, &add.longName));
-            CTStripWhitespace(add.shortName);
-            CTStripWhitespace(add.longName);
-
-            if(!IsNullOrEmptyString(add.shortName) && !IsNullOrEmptyString(add.longName))
-            {
-                DJ_LOG_INFO("Lwiinfo found trust '%s' -> '%s'",
-                        add.shortName,
-                        add.longName);
-                if( FindMapping(mappings, add.shortName) == -1)
-                {
-                    GCE(ceError = CTArrayAppend(mappings,
-                                sizeof(add), &add, 1));
-                    memset(&add, 0, sizeof(add));
-                }
-            }
-
-            line = strtok_r(NULL, "\r\n", &linesaveptr);
-        }
-    }
-    else if(ceError == CENTERROR_COMMAND_FAILED)
-    {
-         DJ_LOG_INFO("Failed to run lwiinfo --details -m. This is expected if the auth daemon is not running");
-         ceError = CENTERROR_SUCCESS;
-    }
-    else
-        GCE(ceError);
 
 cleanup:
-    CT_SAFE_FREE_STRING(commandOutput);
     if(!CENTERROR_IS_OK(ceError))
         FreeDomainMappings(mappings);
     CT_SAFE_FREE_STRING(add.shortName);
     CT_SAFE_FREE_STRING(add.longName);
+    if (pStatus)
+    {
+        LsaFreeStatus(pStatus);
+    }
+    if (hLsa)
+    {
+        LsaCloseServer(hLsa);
+    }
+
+    return ceError;
+}
+
+static CENTERROR
+RestoreMacKeberosFile(
+    void
+    )
+{
+    CENTERROR ceError = CENTERROR_SUCCESS;
+    BOOLEAN   bFileExists = FALSE;
+
+    (void) CTRemoveFile("/Library/Preferences/edu.mit.Kerberos");
+
+    GCE(ceError = CTCheckFileExists("/Library/Preferences/edu.mit.Kerberos.orig", &bFileExists));
+
+    if (bFileExists)
+    {
+        DJ_LOG_VERBOSE("Resstoring original /Library/Preferences/edu.mit.Kerberos");
+
+        GCE(ceError = CTCopyFileWithOriginalPerms("/Library/Preferences/edu.mit.Kerberos.orig",
+                                                  "/Library/Preferences/edu.mit.Kerberos"));
+    }
+
+cleanup:
+    return ceError;
+}
+
+static CENTERROR
+CreateMacKeberosFile(
+    PSTR pszDomainName,
+    PSTR pszRealm)
+{
+    CENTERROR ceError = CENTERROR_SUCCESS;
+    BOOLEAN   bDirExists = FALSE;
+    BOOLEAN   bFileExists = FALSE;
+    FILE    * file = NULL;
+
+    /* Test for typical Mac system directory where edu.mit.Kerberos lives */
+    GCE(ceError = CTCheckDirectoryExists("/Library/Preferences", &bDirExists));
+
+    if (bDirExists)
+    {
+        /* Backup existing file */
+        GCE(ceError = CTCheckFileExists("/Library/Preferences/edu.mit.Kerberos", &bFileExists));
+
+        if (bFileExists)
+        {
+            GCE(ceError = CTCheckFileExists("/Library/Preferences/edu.mit.Kerberos.orig", &bFileExists));
+
+            if (bFileExists)
+            {
+                /* There is already a backup version, delete the edu.mit.Kerberos since we will create
+                   a new one below */
+                (void) CTRemoveFile("/Library/Preferences/edu.mit.Kerberos");
+            }
+            else
+            {
+                DJ_LOG_VERBOSE("Backing up original /Library/Preferences/edu.mit.Kerberos");
+
+                GCE(ceError = CTMoveFile("/Library/Preferences/edu.mit.Kerberos",
+                                         "/Library/Preferences/edu.mit.Kerberos.orig"));
+            }
+        }
+
+        ceError = CTOpenFile("/Library/Preferences/edu.mit.Kerberos", "w", &file);
+        if(!CENTERROR_IS_OK(ceError))
+        {
+            DJ_LOG_ERROR("Unable to open '%s' for writing", "/Library/Preferences/edu.mit.Kerberos");
+            GCE(ceError);
+        }
+
+        GCE(ceError = CTFilePrintf(file, "# WARNING This file is created during Likewise domain join.\n"));
+        GCE(ceError = CTFilePrintf(file, "# Any previous version of edu.mit.Kerberos is backed up to\n"));
+        GCE(ceError = CTFilePrintf(file, "# /Likewise/Preferences/edu.mit.Kerberos.orig\n"));
+        GCE(ceError = CTFilePrintf(file, "# Leaving the current domain will restore the file above.\n"));
+        GCE(ceError = CTFilePrintf(file, "[libdefaults]\n"));
+        GCE(ceError = CTFilePrintf(file, "\tdefault_realm = %s\n", pszRealm));
+        GCE(ceError = CTFilePrintf(file, "\tdns_lookup_kdc = yes\n"));
+        GCE(ceError = CTFilePrintf(file, "[domain_realm]\n"));
+        GCE(ceError = CTFilePrintf(file, "\t.%s = %s\n", pszDomainName, pszRealm));
+        CTCloseFile(file);
+        file = NULL;
+
+        GCE(ceError = CTChangeOwnerAndPermissions("/Library/Preferences/edu.mit.Kerberos",
+                                                   0, 80, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH));
+    }
+
+cleanup:
+
+    if(file != NULL)
+        CTCloseFile(file);
+
     return ceError;
 }
 
@@ -1001,6 +999,9 @@ Krb5LeaveDomain(Krb5Entry *conf)
     GCE(ceError = EnsureStanzaNode(conf, "libdefaults", &libdefaults));
     GCE(ceError = DeleteChildNode(libdefaults, "default_realm", NULL));
 
+    /* Revert changes needed for SSO support for Mac platforms */
+    GCE(ceError = RestoreMacKeberosFile());
+
 cleanup:
     return ceError;
 }
@@ -1013,12 +1014,14 @@ Krb5JoinDomain(Krb5Entry *conf,
     CENTERROR ceError = CENTERROR_SUCCESS;
     Krb5Entry *libdefaults;
     Krb5Entry *realms;
+    Krb5Entry *domain_realm = NULL;
     Krb5Entry *appdefaults;
     Krb5Entry *pamGroup;
     Krb5Entry *httpdGroup;
     Krb5Entry *domainGroup = NULL;
     Krb5Entry *addNode = NULL;
     char *domainUpper = NULL;
+    char *domainLower = NULL;
     char *autoShortDomain = NULL;
     DynamicArray trusts;
     char *mappingString = NULL;
@@ -1065,10 +1068,14 @@ Krb5JoinDomain(Krb5Entry *conf,
                 "RC4-HMAC DES-CBC-MD5 DES-CBC-CRC" ));
     GCE(ceError = SetNodeValue( libdefaults, "dns_lookup_kdc", "true" ));
 
+    GCE(ceError = EnsureStanzaNode(conf, "domain_realm", &domain_realm));
     GCE(ceError = EnsureStanzaNode(conf, "realms", &realms));
     GCE(ceError = CTStrdup(pszDomainName, &domainUpper));
     CTStrToUpper(domainUpper);
     GCE(ceError = CreateGroupNode(conf, 2, domainUpper, &domainGroup));
+    /* Enable SSO for Mac platforms, by creating a suitable /Library/Preferences/edu.mit.Kerberos file */
+    GCE(ceError = CreateMacKeberosFile((PSTR) pszDomainName, domainUpper));
+
     for(i = 0; i < trusts.size; i++)
     {
         DomainMapping *current = ((DomainMapping *)trusts.data) + i;
@@ -1077,6 +1084,15 @@ Krb5JoinDomain(Krb5Entry *conf,
         GCE(ceError = CreateValueNode(conf, 3, "auth_to_local", mappingString, &addNode));
         GCE(ceError = AddChildNode(domainGroup, addNode));
         addNode = NULL;
+
+        CT_SAFE_FREE_STRING(domainUpper);
+        CT_SAFE_FREE_STRING(domainLower);
+        GCE(ceError = CTStrdup(current->longName, &domainUpper));
+        CTStrToUpper(domainUpper);
+        GCE(ceError = CTAllocateStringPrintf(
+                &domainLower, ".%s", current->longName));
+        CTStrToLower(domainLower);
+        GCE(ceError = SetNodeValue(domain_realm, domainLower, domainUpper));
     }
     GCE(ceError = CreateValueNode(conf, 3, "auth_to_local", "DEFAULT", &addNode));
     GCE(ceError = AddChildNode(domainGroup, addNode));
@@ -1105,8 +1121,10 @@ Krb5JoinDomain(Krb5Entry *conf,
     GCE(ceError = SetNodeValue( httpdGroup, "reverse_mappings", mappingString ));
 
 cleanup:
+    CT_SAFE_FREE_STRING(domainLower);
     CT_SAFE_FREE_STRING(mappingString);
     CT_SAFE_FREE_STRING(autoShortDomain);
+    CT_SAFE_FREE_STRING(domainUpper);
     FreeDomainMappings(&trusts);
     FreeKrb5Entry(&domainGroup);
     FreeKrb5Entry(&addNode);
@@ -1280,6 +1298,7 @@ static CENTERROR WriteKrb5Configuration(const char *rootPrefix, const char *file
 {
     CENTERROR ceError = CENTERROR_SUCCESS;
     FILE *file = NULL;
+    char *prefixedPath = NULL;
     char *tempName = NULL;
     char *finalName = NULL;
     char *symtarget = NULL;
@@ -1289,8 +1308,13 @@ static CENTERROR WriteKrb5Configuration(const char *rootPrefix, const char *file
     if(rootPrefix == NULL)
         rootPrefix = "";
 
-    GCE(ceError = CTAllocateStringPrintf(&tempName, "%s%s.new", rootPrefix, filename));
-    GCE(ceError = CTAllocateStringPrintf(&finalName, "%s%s", rootPrefix, filename));
+    GCE(ceError = CTAllocateStringPrintf(&prefixedPath, "%s%s", rootPrefix, filename));
+
+    ceError = CTGetFileTempPath(
+                        prefixedPath,
+                        &finalName,
+                        &tempName);
+    GCE(ceError);
 
     DJ_LOG_INFO("Writing krb5 file %s", finalName);
 
@@ -1326,14 +1350,18 @@ static CENTERROR WriteKrb5Configuration(const char *rootPrefix, const char *file
             finalName = symtarget;
             symtarget = NULL;
         }
+#if defined(MINIMAL_JOIN)
         GCE(ceError = CTCloneFilePerms(finalName, tempName));
-        GCE(ceError = CTBackupFile(finalName));
         GCE(ceError = CTMoveFile(tempName, finalName));
+#else
+        GCE(ceError = CTSafeReplaceFile(finalName, tempName));
+#endif
     }
 
 cleanup:
     if(file != NULL)
         CTCloseFile(file);
+    CT_SAFE_FREE_STRING(prefixedPath);
     CT_SAFE_FREE_STRING(tempName);
     CT_SAFE_FREE_STRING(finalName);
     CT_SAFE_FREE_STRING(symtarget);
@@ -1472,7 +1500,7 @@ static QueryResult QueryKrb5(const JoinProcessOptions *options, LWException **ex
     else
     {
         //Ignore failures from this command
-        DJGuessShortDomainName(options->domainName, &shortName);
+        DJGuessShortDomainName(options->domainName, &shortName, NULL);
     }
     LW_CLEANUP_CTERR(exc, DJModifyKrb5Conf(tempDir, options->joiningDomain,
         options->domainName,
@@ -1523,7 +1551,7 @@ static PSTR GetKrb5Description(const JoinProcessOptions *options, LWException **
     else
     {
         //Ignore failures from this command
-        DJGuessShortDomainName(options->domainName, &shortName);
+        DJGuessShortDomainName(options->domainName, &shortName, NULL);
     }
     LW_CLEANUP_CTERR(exc, DJModifyKrb5Conf(tempDir, options->joiningDomain,
         options->domainName,
