@@ -1,25 +1,31 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
-* ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
-* -*- mode: c, c-basic-offset: 4 -*- */
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright (C) Centeris Corporation 2004-2007
- * Copyright (C) Likewise Software    2007-2008
+ * Copyright Likewise Software    2004-2008
  * All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as 
- * published by the Free Software Foundation; either version 2.1 of 
- * the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU Lesser General Public 
- * License along with this program.  If not, see 
- * <http://www.gnu.org/licenses/>.
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
  */
 
 #include "domainjoin.h"
@@ -250,7 +256,7 @@ DJParseHostsFile(
         CTStripWhitespace(szBuf);
 
         ceError = CTAllocateMemory(sizeof(HOSTSFILELINE),
-                                   (PVOID*)&pHostsLine);
+                                   (PVOID*)(PVOID)&pHostsLine);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
         pHostsLine->pEntry = NULL;
@@ -267,7 +273,7 @@ DJParseHostsFile(
         if(szBuf[0] != '\0')
         {
             ceError = CTAllocateMemory(sizeof(HOSTSFILEENTRY),
-                                       (PVOID*)&pHostsLine->pEntry);
+                                       (PVOID*)(PVOID)&pHostsLine->pEntry);
             BAIL_ON_CENTERIS_ERROR(ceError);
 
 
@@ -290,7 +296,7 @@ DJParseHostsFile(
                 } else {
 
                     ceError = CTAllocateMemory(sizeof(HOSTFILEALIAS),
-                                               (PVOID*)&pAlias);
+                                               (PVOID*)(PVOID)&pAlias);
                     BAIL_ON_CENTERIS_ERROR(ceError);
 
                     ceError = CTAllocateString(pszTmp, &pAlias->pszAlias);
@@ -380,7 +386,7 @@ DJAddAlias(
 
     if (alwaysAdd || !DJEntryHasAlias(pLine->pEntry->pAliasList, pszName)) {
 
-        ceError = CTAllocateMemory(sizeof(HOSTFILEALIAS), (PVOID*)&pAlias);
+        ceError = CTAllocateMemory(sizeof(HOSTFILEALIAS), (PVOID*)(PVOID)&pAlias);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
         ceError = CTAllocateString(pszName, &pAlias->pszAlias);
@@ -489,11 +495,19 @@ DJWriteHostsFileIfModified(
     FILE* fp = NULL;
     PHOSTFILEALIAS pAlias = NULL;
     BOOLEAN bRemoveFile = FALSE;
+    char *tempName = NULL;
+    char *finalName = NULL;
 
-    if (DJHostsFileWasModified(pHostFileLineList)) {
+    ceError = CTGetFileTempPath(
+                        filename,
+                        &finalName,
+                        &tempName);
+    BAIL_ON_CENTERIS_ERROR(ceError);
 
-        DJ_LOG_INFO("Writing out updated %s file", filename);
-        fp = fopen("/etc/hosts.domainjoin", "w");
+    if (DJHostsFileWasModified(pHostFileLineList))
+    {
+        DJ_LOG_INFO("Writing out updated %s file", finalName);
+        fp = fopen(tempName, "w");
         if (fp == NULL) {
             ceError = CTMapSystemError(errno);
             BAIL_ON_CENTERIS_ERROR(ceError);
@@ -538,10 +552,7 @@ DJWriteHostsFileIfModified(
             fp = NULL;
         }
 
-        ceError = CTBackupFile(filename);
-        BAIL_ON_CENTERIS_ERROR(ceError);
-
-        ceError = CTMoveFile("/etc/hosts.domainjoin", filename);
+        ceError = CTSafeReplaceFile(finalName, tempName);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
 #if defined(__LWI_MACOSX__)
@@ -553,7 +564,7 @@ DJWriteHostsFileIfModified(
         bRemoveFile = FALSE;
     }
     else
-        DJ_LOG_INFO("%s file was not modified; not rewriting", filename);
+        DJ_LOG_INFO("%s file was not modified; not rewriting", finalName);
 
 error:
 
@@ -561,7 +572,10 @@ error:
         fclose(fp);
 
     if (bRemoveFile)
-        CTRemoveFile("/etc/hosts.domainjoin");
+        CTRemoveFile(tempName);
+
+    CT_SAFE_FREE_STRING(finalName);
+    CT_SAFE_FREE_STRING(tempName);
 
     return ceError;
 }
@@ -725,7 +739,7 @@ DJReplaceHostnameInMemory(
         CTStripWhitespace(pszDomainName);
         CTStrToLower(pszDomainName);
         ceError = CTAllocateMemory(strlen(pszHostName)+strlen(pszDomainName)+2,
-                                   (PVOID*)&pszCanonicalName);
+                                   (PVOID*)(PVOID)&pszCanonicalName);
         BAIL_ON_CENTERIS_ERROR(ceError);
 
         sprintf(pszCanonicalName, "%s.%s", pszHostName, pszDomainName);
@@ -778,11 +792,11 @@ DJReplaceHostnameInMemory(
             if(pLine == NULL)
             {
                 //We have to create the 127.0.0.1 address
-                ceError = CTAllocateMemory(sizeof(HOSTSFILELINE), (PVOID*)&pCreatedLine);
+                ceError = CTAllocateMemory(sizeof(HOSTSFILELINE), (PVOID*)(PVOID)&pCreatedLine);
                 BAIL_ON_CENTERIS_ERROR(ceError);
 
                 ceError = CTAllocateMemory(sizeof(HOSTSFILEENTRY),
-                                           (PVOID*)&pCreatedLine->pEntry);
+                                           (PVOID*)(PVOID)&pCreatedLine->pEntry);
 
                 ceError = CTAllocateString("127.0.0.1",
                                            &pCreatedLine->pEntry->pszIpAddress);
@@ -826,11 +840,11 @@ DJCopyLine(
         PHOSTSFILELINE *ppDest)
 {
     PHOSTSFILELINE ret = NULL;
-    PHOSTFILEALIAS srcAlias = NULL, *destAlias;
+    PHOSTFILEALIAS srcAlias = NULL, *destAlias = NULL;
     CENTERROR ceError = CENTERROR_SUCCESS;
 
     ceError = CTAllocateMemory(sizeof(HOSTSFILELINE),
-                               (PVOID*)&ret);
+                               (PVOID*)(PVOID)&ret);
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     ret->bModified = pSrc->bModified;
@@ -840,7 +854,7 @@ DJCopyLine(
     if(pSrc->pEntry != NULL)
     {
         ceError = CTAllocateMemory(sizeof(HOSTSFILEENTRY),
-                                   (PVOID*)&ret->pEntry);
+                                   (PVOID*)(PVOID)&ret->pEntry);
         BAIL_ON_CENTERIS_ERROR(ceError);
         ceError = CTDupOrNullStr(pSrc->pEntry->pszIpAddress, &ret->pEntry->pszIpAddress);
         BAIL_ON_CENTERIS_ERROR(ceError);
@@ -854,7 +868,7 @@ DJCopyLine(
     while(srcAlias != NULL)
     {
         ceError = CTAllocateMemory(sizeof(HOSTFILEALIAS),
-                                   (PVOID*)destAlias);
+                                   (PVOID*)(PVOID)destAlias);
         BAIL_ON_CENTERIS_ERROR(ceError);
         ceError = CTDupOrNullStr(srcAlias->pszAlias, &(*destAlias)->pszAlias);
         BAIL_ON_CENTERIS_ERROR(ceError);
