@@ -1,25 +1,31 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
-* ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
-* -*- mode: c, c-basic-offset: 4 -*- */
+ * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
+ * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright (C) Centeris Corporation 2004-2007
- * Copyright (C) Likewise Software    2007-2008
+ * Copyright Likewise Software    2004-2008
  * All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as 
- * published by the Free Software Foundation; either version 2.1 of 
- * the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU Lesser General Public 
- * License along with this program.  If not, see 
- * <http://www.gnu.org/licenses/>.
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
  */
 
 #include "domainjoin.h"
@@ -384,12 +390,20 @@ static CENTERROR WriteSshConfiguration(const char *rootPrefix, struct SshConf *c
     int i;
     char *tempName = NULL;
     char *finalName = NULL;
+    char *prefixedPath = NULL;
     FILE *file = NULL;
     memset(&printedLine, 0, sizeof(printedLine));
 
     DJ_LOG_INFO("Writing ssh configuration for %s", conf->filename);
 
-    BAIL_ON_CENTERIS_ERROR(ceError = CTAllocateStringPrintf(&tempName, "%s%s.new", rootPrefix, conf->filename));
+    BAIL_ON_CENTERIS_ERROR(ceError = CTAllocateStringPrintf(&prefixedPath, "%s%s", rootPrefix, conf->filename));
+
+    ceError = CTGetFileTempPath(
+                        prefixedPath,
+                        &finalName,
+                        &tempName);
+    BAIL_ON_CENTERIS_ERROR(ceError);
+
     ceError = CTOpenFile(tempName, "w", &file);
     if(!CENTERROR_IS_OK(ceError))
     {
@@ -405,15 +419,14 @@ static CENTERROR WriteSshConfiguration(const char *rootPrefix, struct SshConf *c
 
     BAIL_ON_CENTERIS_ERROR(ceError = CTCloseFile(file));
     file = NULL;
-    BAIL_ON_CENTERIS_ERROR(ceError = CTAllocateStringPrintf(&finalName, "%s%s", rootPrefix, conf->filename));
-    BAIL_ON_CENTERIS_ERROR(ceError = CTBackupFile(finalName));
-    BAIL_ON_CENTERIS_ERROR(ceError = CTMoveFile(tempName, finalName));
+    BAIL_ON_CENTERIS_ERROR(ceError = CTSafeReplaceFile(finalName, tempName));
     DJ_LOG_INFO("File moved into place");
 
 error:
     if(file != NULL)
         CTCloseFile(file);
     CTArrayFree(&printedLine);
+    CT_SAFE_FREE_STRING(prefixedPath);
     CT_SAFE_FREE_STRING(tempName);
     CT_SAFE_FREE_STRING(finalName);
     return ceError;
@@ -423,8 +436,8 @@ BOOLEAN FindSshAndConfig(PCSTR rootPrefix, PCSTR sshOrSshd,
         PSTR *sshBinary, PSTR *sshConfig, LWException **exc)
 {
     /* Mac OS X stores the configuration in /etc */
-    /* CSW ssh on Solaris uses /opt/csw/etc */
-    const char *sshConfigPath = "/etc/ssh:/opt/ssh/etc:/usr/local/etc:/etc:/etc/openssh:/usr/openssh/etc:/opt/csw/etc";
+    /* CSW ssh on Solaris uses /opt/csw/etc/ssh */
+    const char *sshConfigPath = "/etc/ssh:/opt/ssh/etc:/usr/local/etc:/etc:/etc/openssh:/usr/openssh/etc:/opt/csw/etc/ssh";
 
     /* Solaris Sparc 10 stores sshd in /usr/lib/ssh */
     /* Experian says their sshd is at /usr/openssh/sbin/sshd */
